<?php


namespace vipclient;

use vipclient\exceptions\ExceptionClient;

/**
 * Class Client
 *
 * @package vipclient
 */
class Client
{
    /** @var Net */
    private $net;
    /** @var string  */
    private $baseUrl = 'https://api.virginiavip.ru/';

    /**
     * Client constructor.
     *
     * @param string $userId
     * @param string $token
     * @throws \Exception
     */
    public function __construct($userId, $token)
    {
        $this->net = new Net(
            [
                'format' => 'json',
                'verify' => false,
            ],
            [
                'X-Auth-User' => $userId,
                'X-Auth-Token' => $token,
            ]
        );
    }

    /**
     * @param string $url
     * @param array|null $params
     * @param array|null $data
     * @param string $method
     * @return mixed
     * @throws \Exception
     */
    public function sendRequest($url, $params = null, $data = null, $method = Net::METHOD_GET)
    {
        if (null !== $params && is_array($params)) {
            $paramsQuery = '?' . http_build_query($params);
        } else {
            $paramsQuery = '';
        }

        $result = $this->net->query($this->baseUrl . $url . $paramsQuery, $data, $method);
        if ($result['status'] === 'error') {
            if (array_key_exists('message', $result['data']) && is_array($result['data']['message'])) {
                $messageText = [];
                foreach ($result['data']['message'] as $message) {
                    if (is_array($message)) {
                        $messageText[] = "({$message['code']}) {$message['description']}";
                    } else {
                        $messageText[] = $message;
                    }
                }
                $messageText = implode(PHP_EOL, $messageText);
            } else {
                $messageText = wp('Unknown error.');
            }
            throw new ExceptionClient($messageText);
        }
        return $result['data'];
    }
}