<?php


namespace vipclient\api;

use vipclient\Net;
use vipclient\order\ExtraService;
use vipclient\order\Order;
use vipclient\order\OrderAutomobile;
use vipclient\order\OrderContact;
use vipclient\order\OrderExtraService;
use vipclient\order\OrderPassenger;

/**
 * Class ApiOrder
 *
 * @package vipclient\api
 */
class ApiOrder extends ApiBase
{
    /**
     * @param Order $data
     * @return mixed
     * @throws \Exception
     */
    public function calc(Order $data)
    {
        $orderData = $this->orderToArray($data);
        $result = [];
        $data = $this->client->sendRequest('order/calc/', null, $orderData, Net::METHOD_POST);
        return $result = $data;
    }

    /**
     * @param Order $data
     * @return mixed
     * @throws \Exception
     */
    public function create(Order $data)
    {
        $orderData = $this->orderToArray($data);
        $result = [];
        return $this->client->sendRequest('order/create/', null, $orderData, Net::METHOD_POST);
    }

    /**
     * @param Order $data
     * @return mixed
     * @throws \Exception
     */
    public function edit(Order $data)
    {
        $orderData = $this->orderToArray($data);
        $result = [];
        return $this->client->sendRequest('order/edit/', null, $orderData, Net::METHOD_POST);
    }

    /**
     * @return array
     * @throws \Exception
     */
    public function extraServices()
    {
        $result = [];
        $data = $this->client->sendRequest('order/extra-services/');
        foreach ($data as $itemData) {
            $item = new ExtraService();
            $item->load($itemData);
            $result[] = $item;
        }
        return $result;
    }

    /**
     * @param int $id
     * @return mixed
     * @throws \Exception
     */
    public function view($id)
    {
        $data = $this->client->sendRequest('order/view/', ['number' => $id]);
        return $this->arrayToOrder($data);
    }

    /**
     * @param int $page
     * @return mixed
     * @throws \Exception
     */
    public function list(int $page = 1)
    {
        $data = $this->client->sendRequest('order/list/', ['page' => $page]);
        return $data;
    }

    /**
     * @param $id
     * @return mixed
     * @throws \Exception
     */
    public function cancel($id)
    {
        $orderData = ['number' => $id];
        $result = [];
        $data = $this->client->sendRequest('order/cancel/', null, $orderData, Net::METHOD_POST);
        if (array_key_exists('confirmationCode', $data)) {
            $orderData['confirmationCode'] = $data['confirmationCode'];
            $data = $this->client->sendRequest('order/cancel/', null, $orderData, Net::METHOD_POST);
        }
        return $data;
    }

    /**
     * @param Order $order
     * @return array
     */
    private function orderToArray(Order $order)
    {
        $result = [
            'number' => $order->number,
            'date' => $order->date,
            'flightData' => [
                'terminal' => $order->flightData->terminal,
                'direction' => $order->flightData->direction,
                'date' => $order->flightData->date,
                'time' => $order->flightData->time,
                'number' => $order->flightData->number,
                'way' => $order->flightData->way,
            ],
            'passengers' => [],
            'automobiles' => [],
            'extraServices' => [],
            'contacts' => [],
        ];

        /** @var OrderPassenger $passenger */
        foreach ($order->passengers as $passenger) {
            $result['passengers'][] = [
                'name' => $passenger->name,
                'lastname' => $passenger->lastname,
                'middlename' => $passenger->middlename,
                'birthday' => $passenger->birthday,
            ];
        }

        /** @var OrderExtraService $extraService */
        foreach ($order->extraServices as $extraService) {
            $extraServiceData = [
                'code' => $extraService->code,
            ];
            empty($extraService->customerText) or $extraServiceData['customerText'] = $extraService->customerText;
            empty($extraService->customerCount) or $extraServiceData['customerCount'] = $extraService->customerCount;

            if ($extraService->child && is_array($extraService->child)) {
                $extraServiceData['child'] = [];
                /** @var OrderExtraService $extraServiceChild */
                foreach ($extraService->child as $extraServiceChild) {
                    $extraServiceChildData = [
                        'code' => $extraServiceChild->code,
                    ];
                    empty($extraServiceChild->customerText) or $extraServiceChildData['customerText'] = $extraServiceChild->customerText;
                    empty($extraServiceChild->customerCount) or $extraServiceChildData['customerCount'] = $extraServiceChild->customerCount;
                    $extraServiceData['child'][] = $extraServiceChildData;
                }
            }

            $result['extraServices'][] = $extraServiceData;
        }

        /** @var OrderAutomobile $automobile */
        foreach ($order->automobiles as $automobile) {
            $result['automobiles'][] = [
                'model' => $automobile->model,
                'number' => $automobile->number,
            ];
        }

        /** @var OrderContact $contact */
        foreach ($order->contacts as $contact) {
            $result['contacts'][] = [
                'name' => $contact->name,
                'lastname' => $contact->lastname,
                'middlename' => $contact->middlename,
                'phone' => $contact->phone,
                'phone2' => $contact->phone2,
                'email' => $contact->email,
            ];
        }

        return $result;
    }

    /**
     * @param array $data
     * @return Order
     */
    private function arrayToOrder($data)
    {
        $order = new Order();
        $order->date = $data['date'];
        $order->number = $data['number'];
        $order->status = $data['status'];
        $order->statusPay = $data['statusPay'];
        $order->payType = $data['payType'];
        $order->flightData->number = $data['flightData']['number'];
        $order->flightData->terminal = $data['flightData']['terminal'];
        $order->flightData->direction = $data['flightData']['direction'];
        $order->flightData->way = $data['flightData']['way'];
        $order->flightData->date = $data['flightData']['date'];
        $order->flightData->time = $data['flightData']['time'];

        $passengers = [];
        foreach ($data['passengers'] as $passengerData) {
            $passenger = new OrderPassenger();
            $passenger->name = $passengerData['name'];
            $passenger->lastname = $passengerData['lastname'];
            $passenger->middlename = $passengerData['middlename'];
            $passenger->birthday = $passengerData['birthday'];
            $passengers[] = $passenger;
        }
        $order->passengers = $passengers;

        $automobiles = [];
        foreach ($data['automobiles'] as $automobileData) {
            $automobile = new OrderAutomobile();
            $automobile->model = $automobileData['model'];
            $automobile->number = $automobileData['number'];
            $automobiles[] = $automobile;
        }
        $order->automobiles = $automobiles;

        $contacts = [];
        foreach ($data['contacts'] as $contactData) {
            $contact = new OrderContact();
            $contact->name = $contactData['name'];
            $contact->lastname = $contactData['lastname'];
            $contact->middlename = $contactData['middlename'];
            $contact->phone = $contactData['phone'];
            $contact->phone2 = $contactData['phone2'];
            $contact->email = $contactData['email'];
            $contacts[] = $contact;
        }
        $order->contacts = $contacts;

        $extraServices = [];
        foreach ($data['extraServices'] as $extraServiceData) {
            $extraService = new OrderExtraService();
            $extraService->code = $extraServiceData['code'];
            array_key_exists('customerCount', $extraServiceData) and $extraService->customerCount = $extraServiceData['customerCount'];
            array_key_exists('customerText', $extraServiceData) and $extraService->customerText = $extraServiceData['customerText'];
            if (array_key_exists('child', $extraServiceData) && is_array($extraServiceData['child'])) {
                $child = [];
                foreach ($extraServiceData['child'] as $extraServiceChildData) {
                    $extraServiceChild = new OrderExtraService();
                    $extraServiceChild->code = $extraServiceChildData['code'];
                    array_key_exists('customerCount', $extraServiceChildData) and $extraServiceChild->customerCount = $extraServiceChildData['customerCount'];
                    array_key_exists('customerText', $extraServiceChildData) and $extraServiceChild->customerText = $extraServiceChildData['customerText'];
                    $child[] = $extraServiceChild;
                }
                $extraService->child = $child;
            }
            $extraServices[] = $extraService;
        }
        $order->extraServices = $extraServices;

        $order->sumTariff = $data['sum']['sumTariff'];
        $order->sumExtraServices = $data['sum']['sumExtraService'];
        $order->sumTotal = $data['sum']['sumTotal'];

        return $order;
    }
}