<?php


namespace vipclient\order;

use vipclient\BaseObject;
use vipclient\exceptions\ExceptionData;

/**
 * Class Order
 *
 * @package vipclient\airport
 * @property string $number
 * @property string $date
 * @property string $status
 * @property string $statusPay
 * @property string $payType
 * @property OrderFlightData $flightData
 * @property array $passengers
 * @property array $automobiles
 * @property array $contacts
 * @property array $extraServices
 * @property float $sumTariff
 * @property float $sumExtraServices
 * @property float $sumTotal
 */
class Order extends BaseObject
{
    /** @var string */
    protected $number;
    /** @var string */
    protected $date;
    /** @var string */
    protected $status;
    /** @var string */
    protected $statusPay;
    /** @var string */
    protected $payType;
    /** @var OrderFlightData  */
    protected $flightData;
    /** @var array  */
    protected $passengers = [];
    /** @var array  */
    protected $automobiles = [];
    /** @var array  */
    protected $contacts = [];
    /** @var array  */
    protected $extraServices = [];

    /** @var float */
    protected $sumTariff;
    /** @var float */
    protected $sumExtraServices;
    /** @var float */
    protected $sumTotal;


    public function __construct()
    {
        $this->flightData = new OrderFlightData();
    }

    /**
     * @param array $data
     */
    public function load(array $data)
    {
        parent::load($data);
    }

    /**
     * @return string
     */
    protected function getNumber()
    {
        return $this->number;
    }

    /**
     * @return string
     */
    protected function getDate()
    {
        return $this->date;
    }

    /**
     * @param $value
     */
    public function setDate($value)
    {
        $this->date = $value;
    }

    /**
     * @param $value
     */
    public function setNumber($value)
    {
        $this->number = $value;
    }

    /**
     * @return OrderFlightData
     */
    protected function getFlightData()
    {
        return $this->flightData;
    }

    /**
     * @return array
     */
    protected function getPassengers()
    {
        return $this->passengers;
    }

    /**
     * @return array
     */
    protected function getAutomobiles()
    {
        return $this->automobiles;
    }

    /**
     * @return array
     */
    protected function getContacts()
    {
        return $this->contacts;
    }

    /**
     * @return array
     */
    protected function getExtraServices()
    {
        return $this->extraServices;
    }

    /**
     * @param array $value
     * @throws ExceptionData
     */
    protected function setPassengers(array $value)
    {
        foreach ($value as $item) {
            if (!($item instanceof OrderPassenger)) {
                throw new ExceptionData('Not valid value for passengers');
            }
        }
        $this->passengers = $value;
    }

    /**
     * @param array $value
     * @throws ExceptionData
     */
    protected function setExtraServices(array $value)
    {
        foreach ($value as $item) {
            if (!($item instanceof OrderExtraService)) {
                throw new ExceptionData('Not valid value for extra service');
            }
        }
        $this->extraServices = $value;
    }

    /**
     * @param array $value
     * @throws ExceptionData
     */
    protected function setAutomobiles(array $value)
    {
        foreach ($value as $item) {
            if (!($item instanceof OrderAutomobile)) {
                throw new ExceptionData('Not valid value for automobile');
            }
        }
        $this->automobiles = $value;
    }

    /**
     * @param array $value
     * @throws ExceptionData
     */
    protected function setContacts(array $value)
    {
        foreach ($value as $item) {
            if (!($item instanceof OrderContact)) {
                throw new ExceptionData('Not valid value for contact');
            }
        }
        $this->contacts = $value;
    }

    /**
     * @param float $value
     */
    protected function setSumTariff($value)
    {
        $this->sumTariff = $value;
    }

    /**
     * @return float
     */
    protected function getSumTariff()
    {
        return $this->sumTariff;
    }

    /**
     * @param float $value
     */
    protected function setSumExtraServices($value)
    {
        $this->sumExtraServices = $value;
    }

    /**
     * @return float
     */
    protected function getSumExtraServices()
    {
        return $this->sumExtraServices;
    }

    /**
     * @param float $value
     */
    protected function setSumTotal($value)
    {
        $this->sumTotal = $value;
    }

    /**
     * @return float
     */
    protected function getSumTotal()
    {
        return $this->sumTotal;
    }

    /**
     * @param string $value
     */
    protected function setStatus($value)
    {
        $this->status = $value;
    }

    /**
     * @return string
     */
    protected function getStatus()
    {
        return $this->status;
    }

    /**
     * @param string $value
     */
    protected function setStatusPay($value)
    {
        $this->statusPay = $value;
    }

    /**
     * @return string
     */
    protected function getStatusPay()
    {
        return $this->statusPay;
    }

    /**
     * @param string $value
     */
    protected function setPayType($value)
    {
        $this->payType = $value;
    }

    /**
     * @return string
     */
    protected function getPayType()
    {
        return $this->payType;
    }

    /**
     * @param $code
     * @return OrderExtraService|null
     */
    public function getExtraServiceByCode($code)
    {
        $result = null;
        /** @var OrderExtraService $extraservice */
        foreach ($this->extraServices as $extraService) {
            if ($extraService->code === $code) {
                $result = $extraService;
                break;
            }
        }
        return $result;
    }

}